"""
Tests for 'for_strings' module.
"""

import for_strings as h


def test ():
	"""
	Main function to launch all tests.
	"""
	test____replace ()


def check_result (result, expected_value, error_message):
	if not result == expected_value:
		raise ValueError (error_message)


def test____replace ():
	result1 = h.replace ('aaaa.py.py', '.py', '.js')
	result2 = h.replace ('aaaa.py.py', '.py', '.js', start_from_right = True)
	result3 = h.replace ('aaaa.py.py', '.py', '.js', amount = 1)
	result4 = h.replace ('aaaa.py.py', '.py', '.js', amount = 1, start_from_right = True)

	check_result (result1, 'aaaa.js.js', 'wrong replace 1')
	check_result (result2, 'aaaa.js.js', 'wrong replace 2')
	check_result (result3, 'aaaa.js.py', 'wrong replace 3')
	check_result (result4, 'aaaa.py.js', 'wrong replace 4')




"""
Functions with shortcuts to help with work with strings.
"""




def reverse (source_text):
	"""
	Reverse a string.
	"""
	text = ''
	for i in source_text:
		text = i + text
	return text




def replace (source_string, text_before, text_after, amount = None, start_from_right = False):
	"""
	Replaces one substring in string to another.
	Analog to string method with same name,
	but also adds ability to change direction, from which to replace substring.
	"""
	if not start_from_right:
		if amount:
			return str.replace(source_string, text_before, text_after, amount)
		else:
			return str.replace(source_string, text_before, text_after)
	else:
		reversed_string	= reverse (source_string)
		reversed_before	= reverse (text_before)
		reversed_after	= reverse (text_after)
		if amount:
			reversed_replaced = str.replace (reversed_string, reversed_before, reversed_after, amount)
		else:
			reversed_replaced = str.replace (reversed_string, reversed_before, reversed_after)
		return reverse (reversed_replaced)



"""
Functions to help working with pathes.
If os.path is not enough.
"""




def make_path_abs_if_rel (path, base_dir):
	"""
	Makes path absolute (according to passed base dir) if it's relative.
	"""
	import os.path

	is_abs = os.path.isabs (path)
	if is_abs:
		return path

	abs_path = os.path.abspath (os.path.join (base_dir, path))
	return abs_path


def get_nested_key (dict_obj, keys = [], default_value = None):
	"""
	a = {1: {2: {3: 4, 5: 6}}}
	get_nested_key (a, [1, 2, 3])       == 4
	get_nested_key (a, [1, 2, 3, 4])    == None
	get_nested_key (a, [1, 2, 3, 4], 8) == 8
	"""
	value = default_value

	err_wrong_key                     = KeyError
	error_unable_get_keys_from_object = TypeError
	possible_errors                   = (err_wrong_key, error_unable_get_keys_from_object)

	try:
		last_nested_dict = dict_obj
		for key in keys:
			last_nested_dict = last_nested_dict [key]
		value = last_nested_dict
	except possible_errors:
		return default_value

	return value



#! /usr/bin/python3
"""
Tests for 'for_paths' module.
"""

import for_paths as h


def test ():
	"""
	Main function to launch all tests.
	"""
	test____return_valid_result ()


def check_result (result, expected_value, error_message):
	if not result == expected_value:
		raise ValueError (error_message)


def test____return_valid_result ():
	a = {1: {2: {3: 4, 5: 6}}}

	res1 = h.get_nested_key (a, [1, 2, 3])
	res2 = h.get_nested_key (a, [1, 2, 3, 4])
	res3 = h.get_nested_key (a, [1, 2, 3, 4], 8)

	check_result (res1, 4,    '1')
	check_result (res2, None, '2')
	check_result (res3, 8,    '3')




"""
Functions with shortcuts to help dealing with lists.
"""




def get (collection, index):
	"""
	Returns or element from collection by index (key), or None if not found.
	"""
	try:
		return collection [index]
	except (IndexError, KeyError, TypeError):
		# TypeError: 'NoneType' object is not subscriptable - when instead of collection, None is passed
		return None




def delete (collection, key):
	"""
	Deletes element by key from collection if it's present.
	"""
	try:
		del collection [key]
	except KeyError:
		pass



"""
Functions to help importing other packages.
"""




def require (rel_or_abs_path, __file__ = None, reload = False):
	"""
	Import a file with absolute or relative path specification
	(relative to caller file (`__file__`)).

	Works similarily to `require()` in Node, but if you pass relative path to it -
	then you should also pass `__file__` variable from calling file.

	Parameters:
		. rel_or_abs_path	: relative path to package or module to be imported.
		. __file__			: path to file, from which import is launched.
							Usually you can pass `__file__` variable from calling file.

	Usage:
		import humhelpers as h
		customsockets = h.require ('/home/username/projects/difficult_project/customsockets.py')
		# or:
		customsockets = h.require ('../difficult_project/customsockets.py', __file__)
		# or:
		customsockets = h.require ('../difficult_project/customsockets.py', __file__, reload = True)
	"""
	import os.path
	import sys

	is_abs = os.path.isabs (rel_or_abs_path)

	if is_abs:
		fullpath	= rel_or_abs_path
	else:
		dir_of_file	= os.path.dirname (__file__)
		fullpath	= os.path.abspath (os.path.join (dir_of_file, rel_or_abs_path))

	path_to_module, filename	= os.path.split		(fullpath)
	module_name, ext			= os.path.splitext	(filename)

	sys.path.insert (0, path_to_module)
	module = __import__ (module_name)

	if reload:
		import imp
		imp.reload (module)

	index_of_path = sys.path.index (path_to_module)
	del sys.path [index_of_path]

	return module




import setuptools

with open("README.txt", "tr") as fh:
    long_description = fh.read()

setuptools.setup(
    name="humhelpers",
    version="0.1.4",
    author="vlad1777d (Vladislav Naumov)",
    author_email="naumovvladislav@mail.ru",
    description="Different shortcuts functions (helpers) for humanity.",
    long_description=long_description,
    url="https://bitbucket.org/vlad1777d/humhelpers",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)




def run_async (coroutine):
	"""
	Asynchronously runs coroutine (function, defind with 'async def') until it completes.
	"""
	import asyncio
	loop = asyncio.get_event_loop	()
	loop.run_until_complete			(coroutine())
